package es.gva.cit.jgdal;

public class GdalColorTable extends JNIBase{
	
	private native int getColorEntryCountNat(long cPtr);
	private native short[] getColorEntryAsRGBNat(long cPtr, int pos);
	
	
	/**
	 * Constructor de ColorTable pasandole como parmetro la referencia al objeto 
	 * GdalColorTable en C
	 * 
	 * @param cPtr	direccin de memoria del objeto 
	 */
	
	public GdalColorTable(long cPtr){
		this.cPtr=cPtr;
	}
	
	/**
	 * Obtiene
	 * @return
	 * @throws GdalException
	 */
	public int getColorEntryCount() throws GdalException{
		if(cPtr <= 0)
			throw new GdalException("Error en getColorEntryCount(). La creacin de la tabla no tuvo exito");
			 	
		return getColorEntryCountNat(cPtr);
	}
	
	/**
	 * Obtiene la entrada de la tabla de color de la posicin pasada por parmetro
	 * y la devuelve en forma de objeto GdalColorEntry.
	 * @param pos Posicin de la entrada de la tabla
	 * @return Objeto GdalColorEntry correspondiente a pos
	 * @throws GdalException
	 */
	public GdalColorEntry getColorEntryAsRGB(int pos) throws GdalException{
		if(cPtr <= 0)
			throw new GdalException("Error en getColorEntryAsRGB(). La creacin de la tabla no tuvo exito");
		GdalColorEntry entry = new GdalColorEntry();
		short[] values =  getColorEntryAsRGBNat(cPtr, pos);
		entry.c1 = values[0];
		entry.c2 = values[1];
		entry.c3 = values[2];
		entry.c4 = values[3];
		if(values == null)
			throw new GdalException("Error en getColorEntryAsRGB(). Posicin de la tabla de color inexistente.");
		return entry;
	}
}