/**********************************************************************
 * $Id: GdalRasterBand.java,v 1.9 2006/10/03 07:05:18 nacho Exp $
 *
 * Name:     GdalRasterBand.java
 * Project:  JGDAL. Interface java to gdal (Frank Warmerdam).
 * Purpose:  Basic Funcions about raster bands. 
 * Author:   Nacho Brodin, brodin_ign@gva.es
 *
 **********************************************************************/
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

package es.gva.cit.jgdal;

/**
 * Representa a una banda simple de la imgen o canal.
 * 
 * @author Nacho Brodin <brodin_ign@gva.es>.<BR> Equipo de desarrollo gvSIG.<BR> http://www.gvsig.gva.es
 * @version 0.0
 * @link http://www.gvsig.gva.es
 */

public class GdalRasterBand extends JNIBase{
	

	private native long getOverviewNat(long cPtr,int i);
	private native long getRasterColorTableNat(long cPtr);
	private native GdalBuffer readRasterNat(long cPtr, 
											int nXOff, int nYOff, int nXSize, int nYSize,
											int BufXSize, int BufYSize,
											int eBufType);
	private native GdalBuffer readRasterWithPaletteNat(long cPtr, 
													int nXOff, int nYOff, int nXSize, int nYSize,
													int BufXSize, int BufYSize,
													int eBufType);
	private native void writeRasterNat(	long cPtr, 
										int nXOff, int nYOff, int nXSize, int nYSize,
										GdalBuffer buffer,
										int eBufType);
	private native double getRasterNoDataValueNat(long cPtr);
	private native String[] getMetadataNat(long cPtr,String pszDomain);
	private native int getRasterColorInterpretationNat(long cPtr);
	
	
	/**
	 * Asigna el identificador de la banda
	 */
	
	public GdalRasterBand(long cPtr){
		
		this.cPtr=cPtr;
		
	}
	
	
	/**
	 * Lee datos de la banda de la imgen
	 * 
	 * @return	Devuelve un vector de bytes con el trozo de raster ledo.
	 * @param nXOff	El desplazamiento del pixel desde la esquina superior derecha
	 * de la banda accedida.  
	 * @param nYOff	El desplazamiento de lnea desde la esquina superior derecha
	 * de la banda accedida. 	
	 * @param nXSize	Ancho de la regin en pixels de la banda que ser accedida
	 * @param nYSize	Altura de la regin en lneas de la banda que ser accedida
	 * @param BufXSize	Ancho del buffer donde la regin de la imgen ser guardada
	 * @param BufYSize	Altura del buffer donde la regin de la imgen ser guardada
	 * @param eBufType		
	 */
	
	 public GdalBuffer readRaster(int nXOff, int nYOff, int nXSize, int nYSize,
	 									int BufXSize, int BufYSize,
										int eBufType)throws GdalException{
	 	
	 	if(cPtr <= 0)
			throw new GdalException("Error en GDALGetRasterBand(). La llamada GDALOpen no tuvo exito");
		
	 	
	 	GdalBuffer buffer = readRasterNat(cPtr, nXOff, nYOff, nXSize, nYSize, BufXSize, BufYSize, eBufType);
	 	
	 	if(buffer!=null)
	 		return buffer;
	 	else 
	 		return null;
	 	
	 }
		
	 /**
	  * Escribe datos en la banda de la imgen
	  * 
	  * @param nXOff	El desplazamiento del pixel desde la esquina superior derecha
	  * de la banda accedida.  
	  * @param nYOff	El desplazamiento de lnea desde la esquina superior derecha
	  * de la banda accedida. 	
	  * @param nXSize	Ancho de la regin en pixels de la banda que ser accedida
	  * @param nYSize	Altura de la regin en lneas de la banda que ser accedida
	  * @param BufXSize	Ancho del buffer donde la regin de la imgen ser guardada
	  * @param BufYSize	Altura del buffer donde la regin de la imgen ser guardada
	  * @param eBufType
	  */
	
	 public void writeRaster(int nXOff, int nYOff, int nXSize, int nYSize, GdalBuffer buf, int eBufType)throws GdalException{
	 	
	 	if(cPtr <= 0)
			throw new GdalException("Error en GDALGetRasterBand(). La llamada GDALOpen no tuvo exito");
	 	
	 	GdalBuffer buffer=new GdalBuffer();
	 	switch(eBufType){
	 		case 0:
				return;
	 		case 1:
				buffer.buffByte=buf.buffByte;
	 			break;
	 		case 2:
	 		case 3:
			case 8:
				buffer.buffShort=buf.buffShort;
	 			break;
	 		case 4:
	 		case 5:
			case 9:
	 			buffer.buffInt=buf.buffInt;
	 			break;
	 		case 6:
	 		case 10:
	 			buffer.buffFloat=buf.buffFloat;
	 			break;
	 		case 7:
			case 11:
				buffer.buffDouble=buf.buffDouble;
				break; 		
	 		case 12:
				return;
	 	}
	 
	 	writeRasterNat(cPtr, nXOff, nYOff, nXSize, nYSize, buffer, eBufType);
	 		 	 	
	 }
	 
	/**
	 *Obtiene el tamao en pixeles de la imgen en el eje de las X
	 *@return Tamao en pixeles del eje X
	 *@throws GdalException 
	 */
	
	public int getRasterBandXSize()throws GdalException{
		
		String msg1="Error en getRasterBandXSize(). La llamada getRasterBand no tuvo exito";
		String msg2="Tamao de banda erroneo devuelto por GetRasterBandXSize";
		
		return baseSimpleFunctions(0,msg1,msg2);
	}
	
	/**
	 *Obtiene el tamao en pixeles de la imgen en el eje de las Y
	 *@return Tamao en pixeles del eje Y
	 *@throws GdalException 
	 */
	
	public int getRasterBandYSize()throws GdalException{
		
		String msg1="Error en getRasterBandYSize(). La llamada getRasterBand no tuvo exito";
		String msg2="Tamao de banda erroneo devuelto por GetRasterBandYSize";
		
		return baseSimpleFunctions(1,msg1,msg2);
	}
	
	
	/**
	 * Devuelve el nmero de overviews que contiene la banda.
	 * @return Nmero de overviews
	 * @throws GdalException 
	 */
	
	public int getOverviewCount()throws GdalException{
		
		String msg1="Error en getOverviewCount(). La llamada getRasterBand no tuvo exito";		
		String msg2="Error al obtener el nmero de overviews";
		
		return baseSimpleFunctions(2,msg1,msg2);
	}
	
	
	/**
	 * Obtiene el overview indicado por el ndice "i".
	 * 
	 * @param i	indice del overview que se quiere recuperar.
	 * @return GdalRasterBand	Banda correspondiente al overview selecccionado
	 * @throws GdalException 
	 */
	
	public GdalRasterBand getOverview(int i)throws GdalException{
		
		long cPtr_ov;
						
		if(cPtr <= 0)
			throw new GdalException("Error en getOverview(). La llamada GDALOpen no tuvo exito");
		
		if(i<0 && i>=this.getOverviewCount())
			throw new GdalException("El overview seleccionado no existe");
		
		cPtr_ov = getOverviewNat(cPtr,i);
		
		if(cPtr_ov < 0)
			throw new GdalException("No ha podido obtenerse el overview seleccionado");
		
		return new GdalRasterBand(cPtr_ov);
	}
	
	
	/**
	 * Devuelve el tamao en X del bloque para esa banda
	 * @return Tamao en pixeles del bloque en el eje X
	 * @throws GdalException 
	 */
	
	public int getBlockXSize()throws GdalException{
		
		String msg1="Error en getBlockXSize(). La llamada getRasterBand no tuvo exito";
		String msg2="Tamao de bloque erroneo devuelto por GetBlockXSize";
		
		return baseSimpleFunctions(3,msg1,msg2);
	}
	
	
	/**
	 * Devuelve el tamao en Y del bloque para esa banda
	 * @return Tamao en pixeles del bloque en el eje Y
	 * @throws GdalException 
	 */
	
	public int getBlockYSize()throws GdalException{
		
		String msg1="Error en getBlockXSize(). La llamada getRasterBand no tuvo exito";
		String msg2="Tamao de bloque erroneo devuelto por GetBlockYSize";
		
		return baseSimpleFunctions(4,msg1,msg2);
	}
	
	/**
	 * Devuelve el tipo de datos de la banda
	 * @return Tamao en pixeles del bloque en el eje Y
	 * @throws GdalException 
	 */
	
	public int getRasterDataType()throws GdalException{
		
		String msg1="Error en getRasterDataType(). La llamada getRasterBand no tuvo exito";
		String msg2="Tipo de dato devuelto por GetRasterDataType erroneo";
		
		return baseSimpleFunctions(9,msg1,msg2);
	}
	
	/**
	 * Obtiene la tabla de color asociada a la imagen
	 */
	public GdalColorTable getRasterColorTable()throws GdalException{
		if(cPtr <= 0)
			throw new GdalException("Error en getRasterColorTable(). La llamada GDALOpen no tuvo exito");
		
		GdalColorTable gct = null;
		long l = getRasterColorTableNat(cPtr);
		if( l>0 )
			gct = new GdalColorTable(l);
		else 
			return null;
		
		return gct;
	}
	
	/**
	 * Lee datos de la banda de la imgen con una paleta asociada
	 * 
	 * @return	Devuelve un vector de bytes con el trozo de raster ledo.
	 * @param nXOff	El desplazamiento del pixel desde la esquina superior derecha
	 * de la banda accedida.  
	 * @param nYOff	El desplazamiento de lnea desde la esquina superior derecha
	 * de la banda accedida. 	
	 * @param nXSize	Ancho de la regin en pixels de la banda que ser accedida
	 * @param nYSize	Altura de la regin en lneas de la banda que ser accedida
	 * @param BufXSize	Ancho del buffer donde la regin de la imgen ser guardada
	 * @param BufYSize	Altura del buffer donde la regin de la imgen ser guardada
	 * @param eBufType		
	 */
	
	 public GdalBuffer readRasterWithPalette(int nXOff, int nYOff, int nXSize, int nYSize,
	 									int BufXSize, int BufYSize,
										int eBufType)throws GdalException{
	 	
	 	if(cPtr <= 0)
			throw new GdalException("Error en GDALGetRasterBand(). La llamada GDALOpen no tuvo exito");
		
	 	
	 	GdalBuffer buffer = readRasterWithPaletteNat(cPtr, nXOff, nYOff, nXSize, nYSize, BufXSize, BufYSize, eBufType);
	 	
	 	if(buffer!=null)
	 		return buffer;
	 	else 
	 		return null;
	 	
	 }
	 
	 /**
	  *Devuelve el valor de NoData
	  */
	 
	 public double getRasterNoDataValue()throws GdalException{
		 if(cPtr <= 0)
				throw new GdalException("Error en GDALGetRasterBand(). La llamada GDALOpen no tuvo exito");
		 return getRasterNoDataValueNat(cPtr);
	 }
	 
	 /**
	  * Obtiene un array de Strings con los metadatos
	  * 
	  * @throws GdalException
	  * @return Array de Strings que corresponden a los metadatos que ofrece la imgen
	  */
	 
	 public String[] getMetadata()throws GdalException{
		 if(cPtr <= 0)
			 throw new GdalException("Error en GDALGetMetadata(). La llamada GDALOpen no tuvo exito");
		 String[] res = getMetadataNat(cPtr,null);
		 if(res == null)
			 return new String[0];
		 else return res;
		 
	 }
	 
	 /**
	 * Obtiene identificador que representa el tipo de banda de color. 
	 * @return	identificador del tipo de banda de color
	 * @throws GdalException
	 */
		
	 public int getRasterColorInterpretation()throws GdalException{
		if(cPtr <= 0)
			throw new GdalException("Error en getColorInterpretationName(). La llamada Open no tuvo exito");
		int bandType = getRasterColorInterpretationNat(cPtr);
		return bandType;		
	 }
}